function [ x1, x2, x3 ] = countTargetPosts(filename)
% [x1, x2, x3] = countTargetPosts(filename)
%   Count the target posts with different types of location information in
%   the file. The temporal bins are hour-of-day(24) as in the timestamps, and 
%   the spatial bins are the 48 continental US plus Washington DC. 
% 
% *Input parameters
%   filename: name of the file containing the spatial and temporal
%   information of target posts. Each line corresponds to one target post,
%   and includes four fields separated by tab: TweetID, timestamp,
%   estimated location, location sources(GPS/LOC/NULL).
%
% *Output parameters
%   x1: counts of target posts with GPS coordinates.
%   x2: counts of target posts with recognizable user profile locations.
%   x3: estimated counts of target posts created from US without location
%   information.

% Number of spatial bins
S = 49;
% Number of temporal bins
T = 24;

% state names and the spatial bins in x are indexed alphabetically (expect
% DC) as in this array.
state = {'Alabama', 'Arizona', 'Arkansas', 'California', ...
    'Colorado', 'Connecticut', 'Delaware', 'Florida', 'Georgia', ...
    'Idaho', 'Illinois', 'Indiana', 'Iowa', 'Kansas', ...
    'Kentucky', 'Louisiana', 'Maine', 'Maryland', 'Massachusetts', ...
    'Michigan', 'Minnesota', 'Mississippi', 'Missouri', 'Montana', ...
    'Nebraska', 'Nevada', 'New Hampshire', 'New Jersey', 'New Mexico', ...
    'New York', 'North Carolina','North Dakota', 'Ohio', 'Oklahoma', ...
    'Oregon', 'Pennsylvania', 'Rhode Island', 'South Carolina', ...
    'South Dakota', 'Tennessee', 'Texas', 'Utah', 'Vermont', 'Virginia',...
    'Washington', 'West Virginia', 'Wisconsin', 'Wyoming',...
    'District of Columbia'};

% Since the tweets without any location information could be created
% outside of US, we need to estimate how many of them were created within 
% the US. We collected a large set of tweets by tracking a list of animal 
% names, kept the ones with GPS coordinates, mapped the coordinates to the
% countries, and computed the fraction of tweets from US over all tweets
% within each hour-of-day.
fractionOfTweetsFromUSA = [
    0.5846
    0.5103
    0.3870
    0.2627
    0.1749
    0.1605
    0.2044
    0.2527
    0.3064
    0.3536
    0.3983
    0.4331
    0.4534
    0.4651
    0.4639
    0.4441
    0.4399
    0.4721
    0.5331
    0.5891
    0.6053
    0.6285
    0.6323
    0.6300
];

x1 = zeros(S, T);
x2 = zeros(S, T);
x3 = zeros(1, T);

indexMap = containers.Map(state, 1:length(state));

fid = fopen(filename);
tline = fgetl(fid);
while ischar(tline)
    fields = textscan(tline, '%s\t%s\t%s\t%s','Delimiter','\t');
    
    % Convert the UTC to central time. Due to the daylight saving time
    % ended at 2am, Nov 6, 2011, we have to determine the differences.
    if (strcmp(fields{2}{1}(5:7),'Nov') && ...
        ((str2num(fields{2}{1}(9:10)) == 6 && str2num(fields{2}{1}(11:13)) > 8) ...
            || str2num(fields{2}{1}(9:10)) > 6) )
        h = mod(str2num(fields{2}{1}(11:13)) - 6, 24) + 1; 
    else
        h = mod(str2num(fields{2}{1}(12:13)) - 5, 24) + 1; 
    end
    if indexMap.isKey(fields{3}{1})
        s = indexMap(fields{3}{1});
        if (strcmp(fields{4}{1}, 'GPS'))
            x1(s, h) = x1(s, h) + 1;
        else
            x2(s, h) = x2(s, h) + 1;
        end
    else
        x3(h) = x3(h) + 1;
    end
    tline = fgetl(fid);
end
fclose(fid);

x3 = fractionOfTweetsFromUSA' .*x3;

