% This script shows how to run Socioscope with an example of monitoring 
% chipmunks roadkill event.

%% Step 1:
% we need to decide the spatial and temporal resolution for the source
% bins. We set the spatial bins to be 48 US continental states plus Washington
% DC, and the temporal bins to be hour-of-day. So S = 49, and T = 24. In 
% total, we have 49*24 = 1176 "source bins." 
% We consider three kinds of detector bins: precise location, potentially noisy 
% location, and missing location. In total, we have T*(2S+1)= 2376 "detector 
% bins."

% Number of spatial bins
S = 49;
% Number of temporal bins
T = 24;

%% Step 2: 
% Generate target post counts in detector bins
[x1, x2, x3] = countTargetPosts('data/chipmunk');
% transform to a vector
x = [x1(:); x2(:); x3(:)];

% Step 3 & 4:
% Estimate and construct following parameters setting:
%   z: the counts of all posts with coordinates (to estimate social media user
%      activity g)
%   P: Transition matrix 
%   W_f: regularization graph for recovering target phenomenon intensity f
%   W_g: regularization graph for recovering social media user activity g
load data/exampleData
% Other variables are loaded for visualization/verification:
%   J: The index to sort the US states from east to west
%   state: US state names ordered by its center from east to west

% Estimate social media user activity g from the z. Given huge volume of posts,
% we don't have population bias issue. Since we are only using the posts with GPS
% information, the "source bins" and "detector bins" have the same size, and 
% the transition matrix is simply an identity matrix (If the scale is of interest,
% multiplying by 3%, the estimated fraction of posts with GPS coordinates). We 
% also pass the "social media user activity" with all-ones vector. The 
% regularization parameter is set with fixed value.
g = socioscope(z, ones(S*T,1), @eta, eye(S*T), W_g, 10^-0.5);
g_matrix = reshape(g, S, T);
imagesc(g_matrix(J,:)); % Fig 2(a) in the Socioscope paper.

% We use the recovered g to estimate the target phenomenon intensity function f.
f  = socioscope(x, g, @eta, P, W_f, 10^-1.5);
% If you want to tune the parameters by cross-validation from a set of candidate
% values, run
% 
% lambda = 10.^(-2:0.5:2);
% [f lambda_opt] = socioscope(x_chipmunk, g, @eta, P, W_f, lambda, 10, 0.1);

f_matrix = reshape(f, S, T);
figure;
imagesc(f_matrix(J, :)); % Fig 4(a) in the Socioscope Paper 

