function [f, lambda_opt] = socioscope(x, g, eta, P, W, lambda, numTrials, hold_out_ratio)

% This function estimates the target phenomenon intensity function f from the 
% target social media post counts.
%
% Input parameters:
%   x: The target post counts in "detector bins" (m*1 vector)
%   g: The social media user activity intensities in "source bins" (n*1 vector)
%   eta: The function handle to the link function eta(f, g). Please include the 
%        gradient with respect to f as the second output argument in the 
%        function to speed up the optimization. 
%   P: The transition matrix from "source bin" to "detector bin" (m*n matrix)
%   W: The adjacency matrix W (n*n matrix) of regularization graph.  
%   lambda: the regularization parameter. Scalar or a vector. If more than one 
%       values are provided in the vector, cross validation will be used to 
%       choose one from them.
%   numTrials: The number of trials of cross validation experiments in the
%   	parameter tuning procedure. Default value: 30.
%   hold_out_ratio: The ratio of counts in hold out set to the whole set in the
%   	parameter tuning procedure. Default value: 0.1.
%
% Output parameters:
%   f: The target phenomenon intensity in source bins (n*1 vector)
%   lambda_opt: The regularization parameter selected by cross validation.

% Compute combinatorial graph Laplacian 
L = diag(sum(W, 2)) - W;

if length(lambda) == 1
    lambda_opt = lambda;
else
    % Run cross validation to select the best lambda.
    
    % Set default parameters if not set by users
    if nargin < 7,
        numTrials = 30;
    end
    if nargin < 8,
        hold_out_ratio = 0.1;
    end

    lambda_opt = tuneParameters(x, g, eta, P, L, lambda, numTrials, hold_out_ratio);
end

% Call the solver of the optimization problem.
f = opt(x, g, eta, P, L, lambda_opt);
